@@ -38,53 +38,70 @@
 
 #define NDIMS 3
 
-#define ERR {if(err!=NC_NOERR){printf("Error at line %d in %s: %s\n", __LINE__,__FILE__, ncmpi_strerror(err));nerrs++;}}
+static int verbose;
+
+#define ERR {if(err!=NC_NOERR){printf("Error at %s:%d : %s\n", __FILE__,__LINE__, ncmpi_strerror(err));nerrs++;}}
 
 static void
 usage(char *argv0)
 {
     char *help =
-    "Usage: %s [-h] | [-q] [-l len] [file_name]\n"
-    "       [-h] Print this help\n"
-    "       [-q] quiet mode\n"
+    "Usage: %s [-h] | [-q] [-k format] [-l len] [file_name]\n"
+    "       [-h] Print help\n"
+    "       [-q] Quiet mode (reports when fail)\n"
+    "       [-k format] file format: 1 for CDF-1, 2 for CDF-2, 3 for NetCDF4,\n"
+    "                                4 for NetCDF4 classic model, 5 for CDF-5\n"
     "       [-l len] size of each dimension of the local array\n"
     "       [filename] output netCDF file name\n";
     fprintf(stderr, help, argv0);
 }
 
-
-/*----< main() >------------------------------------------------------------*/
-int main(int argc, char **argv)
+/*----< pnetcdf_check_mem_usage() >------------------------------------------*/
+/* check PnetCDF library internal memory usage */
+static int
+pnetcdf_check_mem_usage(MPI_Comm comm)
 {
-    extern int optind;
-    char filename[256], str[512];
-    int i, j, k, rank, nprocs, len=0, ncid, bufsize, verbose=1, err, nerrs=0;
+    int err, nerrs=0, rank;
+    MPI_Offset malloc_size, sum_size;
+
+    MPI_Comm_rank(comm, &rank);
+
+    /* print info about PnetCDF internal malloc usage */
+    err = ncmpi_inq_malloc_max_size(&malloc_size);
+    if (err == NC_NOERR) {
+        MPI_Reduce(&malloc_size, &sum_size, 1, MPI_OFFSET, MPI_SUM, 0, MPI_COMM_WORLD);
+        if (rank == 0 && verbose)
+            printf("maximum heap memory allocated by PnetCDF internally is %lld bytes\n",
+                   sum_size);
+
+        /* check if there is any PnetCDF internal malloc residue */
+        err = ncmpi_inq_malloc_size(&malloc_size);
+        MPI_Reduce(&malloc_size, &sum_size, 1, MPI_OFFSET, MPI_SUM, 0, MPI_COMM_WORLD);
+        if (rank == 0 && sum_size > 0)
+            printf("heap memory allocated by PnetCDF internally has %lld bytes yet to be freed\n",
+                   sum_size);
+    }
+    else if (err != NC_ENOTENABLED) {
+        printf("Error at %s:%d: %s\n", __FILE__,__LINE__,ncmpi_strerror(err));
+        nerrs++;
+    }
+    return nerrs;
+}
+
+/*----< pnetcdf_io() >-------------------------------------------------------*/
+static int
+pnetcdf_io(MPI_Comm comm, char *filename, int cmode, int len)
+{
+    char str[512];
+    int i, j, k, rank, nprocs, ncid, bufsize, err, nerrs=0;
     int *buf, psizes[NDIMS], dimids[NDIMS], dimidsT[NDIMS];
     int XYZ_id, XZY_id, YZX_id, YXZ_id, ZYX_id, ZXY_id;
     MPI_Offset gsizes[NDIMS], starts[NDIMS], counts[NDIMS], imap[NDIMS];
     MPI_Offset startsT[NDIMS], countsT[NDIMS];
     MPI_Info info;
 
-    MPI_Init(&argc,&argv);
-    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
-    MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
-
-    /* get command-line arguments */
-    while ((i = getopt(argc, argv, "hql:")) != EOF)
-        switch(i) {
-            case 'q': verbose = 0;
-                      break;
-            case 'l': len = atoi(optarg);
-                      break;
-            case 'h':
-            default:  if (rank==0) usage(argv[0]);
-                      MPI_Finalize();
-                      return 1;
-        }
-    if (argv[optind] == NULL) strcpy(filename, "testfile.nc");
-    else                      snprintf(filename, 256, "%s", argv[optind]);
-
-    len = (len <= 0) ? 2 : len;
+    MPI_Comm_rank(comm, &rank);
+    MPI_Comm_size(comm, &nprocs);
 
     for (i=0; i<NDIMS; i++)
         psizes[i] = 0;
@@ -134,12 +151,12 @@ int main(int argc, char **argv)
     MPI_Info_set(info, "nc_var_align_size", "1");
 
     /* create the file */
-    err = ncmpi_create(MPI_COMM_WORLD, filename, NC_CLOBBER|NC_64BIT_DATA,
-                       info, &ncid);
+    cmode |= NC_CLOBBER;
+    err = ncmpi_create(comm, filename, cmode, info, &ncid);
     if (err != NC_NOERR) {
         printf("Error at line %d in %s: ncmpi_create() file %s (%s)\n",
         __LINE__,__FILE__,filename,ncmpi_strerror(err));
-        MPI_Abort(MPI_COMM_WORLD, -1);
+        MPI_Abort(comm, -1);
         exit(1);
     }
 
@@ -235,6 +252,60 @@ int main(int argc, char **argv)
 
     free(buf);
 
+    return nerrs;
+}
+
+/*----< main() >------------------------------------------------------------*/
+int main(int argc, char **argv)
+{
+    extern int optind;
+    extern char *optarg;
+    char filename[256];
+    int i, rank, len=0, cmode=0, kind=0, nerrs=0;
+
+    MPI_Init(&argc,&argv);
+    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
+
+    verbose = 1;
+
+    /* get command-line arguments */
+    while ((i = getopt(argc, argv, "hqk:l:")) != EOF)
+        switch(i) {
+            case 'q': verbose = 0;
+                      break;
+            case 'k': kind = atoi(optarg);
+                      break;
+            case 'l': len = atoi(optarg);
+                      break;
+            case 'h':
+            default:  if (rank==0) usage(argv[0]);
+                      MPI_Finalize();
+                      return 1;
+        }
+    if (argv[optind] == NULL) strcpy(filename, "testfile.nc");
+    else                      snprintf(filename, 256, "%s", argv[optind]);
+
+    len = (len <= 0) ? 10 : len;
+
+    switch (kind) {
+        case(2): cmode = NC_64BIT_OFFSET;             break;
+        case(3): cmode = NC_NETCDF4;                  break;
+        case(4): cmode = NC_NETCDF4|NC_CLASSIC_MODEL; break;
+        case(5): cmode = NC_64BIT_DATA;               break;
+        default: cmode = 0;
+    }
+
+#ifndef PNETCDF_DRIVER_NETCDF4
+    /* netcdf4 driver is not enabled, skip */
+    if (kind == 3 || kind == 4) {
+        MPI_Finalize();
+        return 0;
+    }
+#endif
+    nerrs += pnetcdf_io(MPI_COMM_WORLD, filename, cmode, len);
+
+    nerrs += pnetcdf_check_mem_usage(MPI_COMM_WORLD);
+
     MPI_Finalize();
     return (nerrs > 0);
 }
