@@ -41,7 +41,7 @@
  *            int fix_var(Y, X) ;
  *
  *    // global attributes:
- *                    :history = "Thu Feb 12 01:28:34 2015\n",
+ *                    :history = "Mon Aug 13 21:27:48 2018" ;
  *       "" ;
  *    data:
  *
@@ -126,6 +126,7 @@ int main(int argc, char** argv)
         /* add a global attribute: a time stamp at rank 0 */
         time_t ltime = time(NULL); /* get the current calendar time */
         asctime_r(localtime(&ltime), str_att);
+        sprintf(str_att, "Mon Aug 13 21:27:48 2018");
 
         /* make sure the time string are consistent among all processes */
         MPI_Bcast(str_att, 256, MPI_CHAR, 0, MPI_COMM_WORLD);
