@@ -99,6 +99,11 @@ ncmpii_error_mpi2nc(int mpi_errorcode, char *msg);
 extern int
 ncmpii_error_posix2nc(char *err_msg);
 
+#ifdef ENABLE_ADIOS
+extern int
+ncmpii_error_adios2nc(int adios_err, char *err_msg);
+#endif
+
 extern int
 ncmpii_check_name(const char *name, int file_ver);
 
@@ -230,5 +235,12 @@ int hash_map_free(hash_map *map);
 int hash_map_add(hash_map *map, char *key, int val);
 int hash_map_find(hash_map *map, char *key, int *val);
 
+#ifndef HAVE_STRDUP
+extern char *strdup(const char *s);
+#endif
+#ifndef HAVE_STRCASECMP
+extern int strcasecmp(const char *s1, const char *s2);
+#endif
+
 #endif
 
