@@ -89,7 +89,7 @@ int ncbbio_put_list_init(NC_bb *ncbbp) {
 static int ncbbio_put_list_resize(NC_bb *ncbbp)
 {
     int i;
-    size_t nsize;
+    ssize_t nsize;
     void *ptr;
     NC_bb_put_list *lp = &(ncbbp->putlist);
 
