@@ -30,7 +30,7 @@ static const char ncmagic2[] = {'C', 'D', 'F', 0x02};
 static const char ncmagic5[] = {'C', 'D', 'F', 0x05};
 
 /*----< hdr_put_NC_name() >--------------------------------------------------*/
-inline static int
+static int
 hdr_put_NC_name(bufferinfo *pbp,
                 const char *name)
 {
@@ -56,11 +56,11 @@ hdr_put_NC_name(bufferinfo *pbp,
     if (err != NC_NOERR) return err;
 
     /* copy namestring */
-    return ncmpix_pad_putn_text(&pbp->pos, (MPI_Offset)nchars, name);
+    return ncmpix_pad_putn_text((void **)(&pbp->pos), (MPI_Offset)nchars, name);
 }
 
 /*----< hdr_put_NC_dim() >---------------------------------------------------*/
-inline static int
+static int
 hdr_put_NC_dim(bufferinfo   *pbp,
                const NC_dim *dimp)
 {
@@ -90,7 +90,7 @@ hdr_put_NC_dim(bufferinfo   *pbp,
 }
 
 /*----< hdr_put_NC_dimarray() >----------------------------------------------*/
-inline static int
+static int
 hdr_put_NC_dimarray(bufferinfo        *pbp,
                     const NC_dimarray *ncap)
 {
@@ -147,7 +147,7 @@ hdr_put_NC_dimarray(bufferinfo        *pbp,
 /*
  * Put the values of an attribute
  */
-inline static int
+static int
 hdr_put_NC_attrV(bufferinfo    *pbp,
                  const NC_attr *attrp)
 {
@@ -188,7 +188,7 @@ hdr_put_NC_attrV(bufferinfo    *pbp,
 }
 
 /*----< hdr_put_NC_attr() >--------------------------------------------------*/
-inline static int
+static int
 hdr_put_NC_attr(bufferinfo    *pbp,
                 const NC_attr *attrp)
 {
@@ -228,7 +228,7 @@ hdr_put_NC_attr(bufferinfo    *pbp,
 }
 
 /*----< hdr_put_NC_attrarray() >---------------------------------------------*/
-inline static int
+static int
 hdr_put_NC_attrarray(bufferinfo         *pbp,
                      const NC_attrarray *ncap)
 {
@@ -455,15 +455,15 @@ ncmpio_hdr_put_NC(NC *ncp, void *buf)
     /* copy "magic", 4 characters */
     if (ncp->format == 5) {
         putbuf.version = 5;
-        status = ncmpix_putn_text(&putbuf.pos, sizeof(ncmagic5), ncmagic5);
+        status = ncmpix_putn_text((void **)(&putbuf.pos), sizeof(ncmagic5), ncmagic5);
     }
     else if (ncp->format == 2) {
         putbuf.version = 2;
-        status = ncmpix_putn_text(&putbuf.pos, sizeof(ncmagic2), ncmagic2);
+        status = ncmpix_putn_text((void **)(&putbuf.pos), sizeof(ncmagic2), ncmagic2);
     }
     else {
         putbuf.version = 1;
-        status = ncmpix_putn_text(&putbuf.pos, sizeof(ncmagic1), ncmagic1);
+        status = ncmpix_putn_text((void **)(&putbuf.pos), sizeof(ncmagic1), ncmagic1);
     }
     if (status != NC_NOERR) return status;
 
@@ -526,7 +526,8 @@ int ncmpio_write_header(NC *ncp)
     MPI_Comm_rank(ncp->comm, &rank);
     if (rank == 0) { /* only root writes to file header */
         MPI_Status mpistatus;
-        void *buf = NCI_Malloc((size_t)ncp->xsz); /* header's write buffer */
+        size_t bufLen = _RNDUP(ncp->xsz, X_ALIGN);
+        void *buf = NCI_Malloc(bufLen); /* header's write buffer */
 
         /* copy header object to write buffer */
         status = ncmpio_hdr_put_NC(ncp, buf);
@@ -537,7 +538,11 @@ int ncmpio_write_header(NC *ncp)
         }
 
 #ifdef _USE_MPI_GET_COUNT
-        /* explicitly initialize mpistatus object to 0, see comments below */
+        /* explicitly initialize mpistatus object to 0. For zero-length read,
+         * MPI_Get_count may report incorrect result for some MPICH version,
+         * due to the uninitialized MPI_Status object passed to MPI-IO calls.
+         * Thus we initialize it above to work around.
+         */
         memset(&mpistatus, 0, sizeof(MPI_Status));
 #endif
         TRACE_IO(MPI_File_write_at)(fh, 0, buf, (int)ncp->xsz, MPI_BYTE, &mpistatus);
