@@ -261,9 +261,9 @@ int ncmpio_subfile_partition(NC *ncp)
     /* NOTE: the following "for loop" should be before NC_begins() */
 
     /* adjust the hints to be used by PnetCDF; use the same value in master */
-    ncp->ncp_sf->h_align = ncp->h_align;
-    ncp->ncp_sf->v_align = ncp->v_align;
-    ncp->ncp_sf->r_align = ncp->r_align;
+    ncp->ncp_sf->h_align    = ncp->h_align;
+    ncp->ncp_sf->fx_v_align = ncp->fx_v_align;
+    ncp->ncp_sf->r_align    = ncp->r_align;
 
     for(i=0; i<ncp->vars.ndefined; i++) { /* traverse all variables */
         NC_var **vpp = ncp->vars.value;
