@@ -2,7 +2,7 @@
  *  Copyright (C) 2014, Northwestern University and Argonne National Laboratory
  *  See COPYRIGHT notice in top-level directory.
  */
-/* $Id: pnetcdf_version.c 3341 2017-08-25 22:29:08Z wkliao $ */
+/* $Id$ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -11,7 +11,7 @@
 
 typedef enum {
     Version_number = 0,
-    Date           = 1, 
+    Date           = 1,
     Patches        = 2,
     Configure_args = 3,
     Compilers      = 4,
@@ -72,7 +72,7 @@ int main( int argc, char *argv[] )
         printf( "PnetCDF Version:    \t%s\n", PNETCDF_VERSION);
     }
     if (flags[Date]) {
-        printf( "PnetCDF Release date:\t19 Dec 2017\n");
+        printf( "PnetCDF Release date:\tFebruary 21, 2022\n");
     }
     if (flags[Configure_args]) {
         printf( "PnetCDF configure: \t%s\n", CONFIGURE_ARGS_CLEAN);
