@@ -120,36 +120,33 @@ static
 int clear_file_contents(int ncid, int *varid)
 {
     int i, err, rank, nerrs=0;
+    MPI_Offset start[2], count[2];
     long long *w_buffer = (long long*) malloc(NY*NX * sizeof(long long));
-    int bb_enabled=0;
-
-    {
-        int flag;
-        char hint[MPI_MAX_INFO_VAL];
-        MPI_Info infoused;
-
-        ncmpi_inq_file_info(ncid, &infoused);
-        MPI_Info_get(infoused, "nc_burst_buf", MPI_MAX_INFO_VAL - 1, hint, &flag);
-        if (flag && strcasecmp(hint, "enable") == 0)
-            bb_enabled = 1;
-        MPI_Info_free(&infoused);
-    }
 
     for (i=0; i<NY*NX; i++) w_buffer[i] = -1;
 
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
 
+    start[0] = start[1] = count[0] = count[1] = 0;
+    if (rank == 0) { /* only rank 0 writes */
+        count[0] = NY;
+        count[1] = NX;
+    }
+
     for (i=0; i<4; i++) {
-        err = ncmpi_put_var_longlong_all(ncid, varid[i], w_buffer);
+        /* cannot use var APIs for record variables */
+        err = ncmpi_iput_vara_longlong(ncid, varid[i], start, count, w_buffer, NULL);
         CHECK_ERR
     }
+    err = ncmpi_wait_all(ncid, NC_REQ_ALL, NULL, NULL);
+    CHECK_ERR
+
     free(w_buffer);
 
-    // Flush the log to prevent new value being skipped due to overlaping domain
-    if (bb_enabled) {
-        err = ncmpi_flush(ncid);
-        CHECK_ERR
-    }
+    /* When using burst buffering, flush the log to prevent new value being
+     * skipped due to overlaping domain
+     */
+    err = ncmpi_flush(ncid); CHECK_ERR
 
     return nerrs;
 }
