@@ -7,7 +7,7 @@
 /* $Id$ */
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- * This program tests if the correct error codes are returns given various
+ * This program tests if the correct error codes are returned given various
  * create/open modes.
  *
  * NC_EINVAL_CMODE should be returned when creating a file using
@@ -17,7 +17,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h> /* strcpy() */
+#include <string.h> /* strdup() */
 #include <libgen.h> /* basename() */
 #include <unistd.h> /* unlink(), access() */
 #include <mpi.h>
@@ -53,19 +53,17 @@ int check_modes(char *filename)
     MPI_Barrier(MPI_COMM_WORLD);
 
     /* create a new file and test various cmodes ----------------------------*/
-    cmode = NC_CLOBBER;
 
     /* It is illegal to use both NC_64BIT_OFFSET and NC_64BIT_DATA together */
-    cmode |= NC_64BIT_OFFSET | NC_64BIT_DATA;
-
+    cmode = NC_CLOBBER | NC_64BIT_OFFSET | NC_64BIT_DATA;
     err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid);
     EXP_ERR(NC_EINVAL_CMODE)
 
     /* The file should not be created */
     if (rank == 0) {
         if (access(path, F_OK) == 0) {
-            printf("Error at line %d in %s: file (%s) should not be created\n",
-                   __LINE__,__FILE__, filename);
+            printf("Error at %s:%d : file (%s) should not be created\n",
+                   __FILE__,__LINE__, filename);
             nerrs++;
             /* delete the file and ignore error */
             unlink(path);
@@ -74,6 +72,44 @@ int check_modes(char *filename)
     }
     MPI_Barrier(MPI_COMM_WORLD);
 
+#ifdef ENABLE_NETCDF4
+    /* It is illegal to use both NC_64BIT_OFFSET and NC_NETCDF4 together */
+    cmode = NC_CLOBBER | NC_64BIT_OFFSET | NC_NETCDF4;
+    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid);
+    EXP_ERR(NC_EINVAL_CMODE)
+
+    /* The file should not be created */
+    if (rank == 0) {
+        if (access(path, F_OK) == 0) {
+            printf("Error at %s:%d : file (%s) should not be created\n",
+                   __FILE__,__LINE__, filename);
+            nerrs++;
+            /* delete the file and ignore error */
+            unlink(path);
+        }
+        /* else : file does not exist */
+    }
+    MPI_Barrier(MPI_COMM_WORLD);
+
+    /* It is illegal to use both NC_64BIT_DATA and NC_NETCDF4 together */
+    cmode = NC_CLOBBER | NC_64BIT_DATA | NC_NETCDF4;
+    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid);
+    EXP_ERR(NC_EINVAL_CMODE)
+
+    /* The file should not be created */
+    if (rank == 0) {
+        if (access(path, F_OK) == 0) {
+            printf("Error at %s:%d : file (%s) should not be created\n",
+                   __FILE__,__LINE__, filename);
+            nerrs++;
+            /* delete the file and ignore error */
+            unlink(path);
+        }
+        /* else : file does not exist */
+    }
+    MPI_Barrier(MPI_COMM_WORLD);
+#endif
+
     /* Collectively opening a non-existing file for read, expect error code
      * NC_ENOENT on all processes */
     err = ncmpi_open(MPI_COMM_WORLD, filename, NC_NOWRITE, MPI_INFO_NULL, &ncid);
@@ -145,8 +181,8 @@ int check_modes(char *filename)
 
 int main(int argc, char** argv)
 {
-    char filename[256];
-    int rank, err, nerrs=0;
+    char *filename=NULL;
+    int len, rank, err, nerrs=0;
 
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
@@ -156,9 +192,11 @@ int main(int argc, char** argv)
         MPI_Finalize();
         return 1;
     }
-    if (argc == 2) snprintf(filename, 256, "%s", argv[1]);
-    else           strcpy(filename, "testfile.nc");
-    MPI_Bcast(filename, 256, MPI_CHAR, 0, MPI_COMM_WORLD);
+    if (argc == 2) filename = strdup(argv[1]);
+    else           filename = strdup("testfile.nc");
+    len = strlen(filename) + 1;
+    MPI_Bcast(&len, 1, MPI_INT, 0, MPI_COMM_WORLD);
+    MPI_Bcast(filename, len, MPI_CHAR, 0, MPI_COMM_WORLD);
 
     if (rank == 0) {
         char *cmd_str = (char*)malloc(strlen(argv[0]) + 256);
@@ -191,6 +229,7 @@ int main(int argc, char** argv)
         if (nerrs) printf(FAIL_STR,nerrs);
         else       printf(PASS_STR);
     }
+    free(filename);
 
     MPI_Finalize();
     return (nerrs > 0);
