@@ -31,32 +31,23 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <strings.h> /* strcasecmp() */
 #include <libgen.h> /* basename() */
 #include <pnetcdf.h>
 
 #include <testutils.h>
 
 static
-int test_cdf2(char *filename)
+int test_cdf12(char *filename, int bb_enabled, int cmode)
 {
-    int err, nerrs=0, ncid, vid, dimid;
+    int err, nerrs=0, ncid, vid, fvid, dvid, dimid;
     unsigned char uc[1];
     signed char sc[1];
     int si[1];
-    int bb_enabled=0;
+    double dbuf;
 
-    err = ncmpi_create(MPI_COMM_WORLD, filename, NC_CLOBBER, MPI_INFO_NULL, &ncid); CHECK_ERR
-
-    {
-        int flag;
-        char hint[MPI_MAX_INFO_VAL];
-        MPI_Info infoused;
-        ncmpi_inq_file_info(ncid, &infoused);
-        MPI_Info_get(infoused, "nc_burst_buf", MPI_MAX_INFO_VAL - 1, hint, &flag);
-        if (flag && strcasecmp(hint, "enable") == 0)
-            bb_enabled = 1;
-        MPI_Info_free(&infoused);
-    }
+    cmode |= NC_CLOBBER;
+    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid); CHECK_ERR
 
     /* for CDF-1 and CDF-2, a special case is made: there is no NC_ERANGE
      * error can occur converting between NC_BYTE and unsigned char.
@@ -81,8 +72,34 @@ int test_cdf2(char *filename)
         nerrs++;
     }
 
+    /* expect NC_ERANGE */
+    dbuf=NC_MAX_DOUBLE/2.0;
+    err = ncmpi_put_att_double(ncid, NC_GLOBAL, "attf", NC_FLOAT, 1, &dbuf);
+    EXP_ERR(NC_ERANGE)
+
+    /* add a new attribute of NC_DOUBLE with a value > NC_MAX_FLOAT */
+    err = ncmpi_put_att_double(ncid, NC_GLOBAL, "attd", NC_DOUBLE, 1, &dbuf);
+    CHECK_ERR
+
+#if defined(PNETCDF_ERANGE_FILL) && PNETCDF_ERANGE_FILL == 1
+    if (! (cmode & NC_NETCDF4)) {
+        float fbuf;
+        /* read back attf and expect NC_FILL_FLOAT */
+        dbuf = 0.0;
+        err = ncmpi_get_att_double(ncid, NC_GLOBAL, "attf", &dbuf); CHECK_ERR
+        if (dbuf != NC_FILL_FLOAT) {
+            printf("Error at line %d: unexpected read value %f (expecting %f)\n",__LINE__,dbuf,NC_FILL_FLOAT);
+            nerrs++;
+        }
+        /* read back attd as float and expect NC_ERANGE */
+        err = ncmpi_get_att_float(ncid, NC_GLOBAL, "attd", &fbuf); EXP_ERR(NC_ERANGE)
+    }
+#endif
+
     err = ncmpi_def_dim(ncid, "x", 1, &dimid); CHECK_ERR
     err = ncmpi_def_var(ncid, "var_byte", NC_BYTE, 1, &dimid, &vid); CHECK_ERR
+    err = ncmpi_def_var(ncid, "var_flt", NC_FLOAT, 1, &dimid, &fvid); CHECK_ERR
+    err = ncmpi_def_var(ncid, "var_dbl", NC_DOUBLE, 1, &dimid, &dvid); CHECK_ERR
     err = ncmpi_enddef(ncid); CHECK_ERR
 
     /* No NC_ERANGE should be returned for CDF-1 and 2 */
@@ -109,11 +126,9 @@ int test_cdf2(char *filename)
     si[0] = -129;
     err = ncmpi_put_var_int_all(ncid, vid, si);
     if (bb_enabled) {
-        if (err == NC_NOERR){
-            err = ncmpi_flush(ncid);
-        }
+        CHECK_ERR
+        err = ncmpi_flush(ncid);
     }
-
     EXP_ERR(NC_ERANGE)
 
     if (si[0] != -129) { /* check if put buffer content is altered */
@@ -124,13 +139,10 @@ int test_cdf2(char *filename)
     /* expect NC_ERANGE */
     si[0] = 256;
     err = ncmpi_put_var_int_all(ncid, vid, si);
-
     if (bb_enabled) {
-        if (err == NC_NOERR){
-            err = ncmpi_flush(ncid);
-        }   
+        CHECK_ERR
+        err = ncmpi_flush(ncid);
     }
-
     EXP_ERR(NC_ERANGE)
 
     if (si[0] != 256) { /* check if put buffer content is altered */
@@ -148,31 +160,48 @@ int test_cdf2(char *filename)
         nerrs++;
     }
 
+    /* expect NC_ERANGE */
+    dbuf = NC_MAX_DOUBLE/2.0;
+    err = ncmpi_put_var_double_all(ncid, fvid, &dbuf);
+    if (bb_enabled) {
+        CHECK_ERR
+        err = ncmpi_flush(ncid);
+    }
+    EXP_ERR(NC_ERANGE)
+
+    /* write a value > NC_MAX_FLOAT */
+    err = ncmpi_put_var_double_all(ncid, dvid, &dbuf); CHECK_ERR
+
+#if defined(PNETCDF_ERANGE_FILL) && PNETCDF_ERANGE_FILL == 1
+    if (! (cmode & NC_NETCDF4)) {
+        float fbuf;
+        /* read back and expect NC_FILL_FLOAT */
+        dbuf = 0.0;
+        err = ncmpi_get_var_double_all(ncid, fvid, &dbuf); CHECK_ERR
+        if (dbuf != NC_FILL_FLOAT) {
+            printf("Error at line %d: unexpected read value %f (expecting %f)\n",__LINE__,dbuf,NC_FILL_FLOAT);
+            nerrs++;
+        }
+
+        /* read back dvid as float and expect NC_ERANGE */
+        err = ncmpi_get_var_float_all(ncid, dvid, &fbuf); EXP_ERR(NC_ERANGE)
+    }
+#endif
+
     err = ncmpi_close(ncid); CHECK_ERR
 
     return nerrs;
 }
 
 static
-int test_cdf5(char *filename)
+int test_cdf345(char *filename, int bb_enabled, int cmode)
 {
     int err, nerrs=0, ncid, uc_vid, sc_vid, dimid;
     unsigned char uc[1];
     signed char sc[1];
-    int bb_enabled=0;
 
-    err = ncmpi_create(MPI_COMM_WORLD, filename, NC_CLOBBER|NC_64BIT_DATA, MPI_INFO_NULL, &ncid); CHECK_ERR
-
-    {
-        int flag;
-        char hint[MPI_MAX_INFO_VAL];
-        MPI_Info infoused;
-        ncmpi_inq_file_info(ncid, &infoused);
-        MPI_Info_get(infoused, "nc_burst_buf", MPI_MAX_INFO_VAL - 1, hint, &flag);
-        if (flag && strcasecmp(hint, "enable") == 0)
-            bb_enabled = 1;
-        MPI_Info_free(&infoused);
-    }
+    cmode |= NC_CLOBBER;
+    err = ncmpi_create(MPI_COMM_WORLD, filename, cmode, MPI_INFO_NULL, &ncid); CHECK_ERR
 
     /* CDF-5 considers NC_BYTE a signed 1-byte integer and NC_UBYTE an
      * unsigned 1-byte integer. The special case in CDF-2 for skipping
@@ -201,23 +230,18 @@ int test_cdf5(char *filename)
 
     sc[0] = -1; /* in CDF-5, put -1 to an uchar should result in NC_ERANGE */
     err = ncmpi_put_var_schar_all(ncid, uc_vid, sc);
-    if (bb_enabled){
-        if (err == NC_NOERR){
-            err = ncmpi_flush(ncid);
-        }
+    if (bb_enabled) {
+        CHECK_ERR
+        err = ncmpi_flush(ncid);
     }
-
     EXP_ERR(NC_ERANGE)
 
     uc[0] = 255; /* in CDF-5, put 255 to a schar should result in NC_ERANGE */
     err = ncmpi_put_var_uchar_all(ncid, sc_vid, uc);
-
-    if (bb_enabled){
-        if (err == NC_NOERR){
-            err = ncmpi_flush(ncid);
-        }
+    if (bb_enabled) {
+        CHECK_ERR
+        err = ncmpi_flush(ncid);
     }
-
     EXP_ERR(NC_ERANGE)
 
     sc[0] = -1;
@@ -232,8 +256,8 @@ int test_cdf5(char *filename)
 
 int main(int argc, char* argv[])
 {
-    char filename[256];
-    int err, nerrs=0, rank;
+    char filename[256], *hint_value;
+    int err, nerrs=0, rank, bb_enabled=0;
 
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
@@ -254,8 +278,21 @@ int main(int argc, char* argv[])
         free(cmd_str);
     }
 
-    nerrs += test_cdf2(filename);
-    nerrs += test_cdf5(filename);
+    /* check whether burst buffering is enabled */
+    if (inq_env_hint("nc_burst_buf", &hint_value)) {
+        if (strcasecmp(hint_value, "enable") == 0) bb_enabled = 1;
+        free(hint_value);
+    }
+
+    nerrs += test_cdf12(filename, bb_enabled, 0);
+    nerrs += test_cdf12(filename, bb_enabled, NC_64BIT_OFFSET);
+#if ENABLE_NETCDF4
+    if (!bb_enabled) {
+        nerrs += test_cdf12(filename, bb_enabled, NC_NETCDF4 | NC_CLASSIC_MODEL);
+        nerrs += test_cdf345(filename, bb_enabled, NC_NETCDF4);
+    }
+#endif
+    nerrs += test_cdf345(filename, bb_enabled, NC_64BIT_DATA);
 
     /* check if PnetCDF freed all internal malloc */
     MPI_Offset malloc_size, sum_size;
