@@ -237,8 +237,15 @@ int main(int argc, char **argv) {
     err = ncmpi_def_var(ncid, "dummy_rec", NC_BYTE, 2, dimids, &varid2); CHECK_ERR
     err = ncmpi_def_dim(ncid, "FIX_DIM", 2, &dimids[0]); CHECK_ERR
     err = ncmpi_def_var(ncid, "fix_var", NC_INT, 2, dimids, &varid1); CHECK_ERR
+    err = ncmpi_set_fill(ncid, NC_FILL, NULL); CHECK_ERR /* enable fill mode */
     err = ncmpi_enddef(ncid); CHECK_ERR
 
+    /* fill 2 records with default fill values */
+    err = ncmpi_fill_var_rec(ncid, varid0, 0); CHECK_ERR
+    err = ncmpi_fill_var_rec(ncid, varid0, 1); CHECK_ERR
+    err = ncmpi_fill_var_rec(ncid, varid2, 0); CHECK_ERR
+    err = ncmpi_fill_var_rec(ncid, varid2, 1); CHECK_ERR
+
     /* create a file type for the record variable */
     int *array_of_blocklengths=(int*) malloc(count[0]*sizeof(int));
     MPI_Aint *array_of_displacements=(MPI_Aint*) malloc(count[0]*sizeof(MPI_Aint));
@@ -273,8 +280,10 @@ int main(int argc, char **argv) {
     expected_put_size = buftype_size;
 
     /* for writing a record variable, root process will update numrec to the
-     * file header */
+     * file header, However, because the first 2 records have been filled
+     * above, root process need not write to file header.
     if (rank == 0) expected_put_size += (format == NC_FORMAT_CDF5) ? 8 : 4;
+     */
     if (expected_put_size != new_put_size - put_size) {
         printf("Error at line %d in %s: unexpected put size (%lld) reported, expecting %d\n",
                __LINE__,__FILE__,new_put_size-put_size, expected_put_size);
@@ -442,13 +451,19 @@ int main(int argc, char **argv) {
     MPI_Type_commit(&ghost_buftype);
 
     MPI_Type_free(&rec_filetype);
+
+    /* construct rec_filetype for dummy_rec of type NC_BYTE */
+    for (i=0; i<count[0]; i++) {
+        array_of_blocklengths[i] = count[1];
+        array_of_displacements[i] = start[1]*sizeof(signed char) + recsize * i;
+    }
     MPI_Type_create_hindexed(2, array_of_blocklengths, array_of_displacements,
                              MPI_SIGNED_CHAR, &rec_filetype);
     MPI_Type_commit(&rec_filetype);
 
     signed char *schar_buf;
     schar_buf = (signed char*) malloc(array_of_sizes[0]*array_of_sizes[1]);
-    for (i=0; i<array_of_sizes[0]*array_of_sizes[1]; i++) schar_buf[i] = i;
+    for (i=0; i<array_of_sizes[0]*array_of_sizes[1]; i++) schar_buf[i] = i + rank*10;
 
     /* write to dummy_rec */
     err = ncmpi_put_vard_all(ncid, varid2, rec_filetype, schar_buf, 1,
@@ -460,7 +475,7 @@ int main(int argc, char **argv) {
                              ghost_buftype); CHECK_ERR
     for (i=0; i<array_of_sizes[0]; i++)
     for (j=0; j<array_of_sizes[1]; j++) {
-        signed char expected = i*array_of_sizes[1] + j;
+        signed char expected = i*array_of_sizes[1] + j + rank*10;
         if (i<2 || i >= 4)         expected = -1;
         else if (j<2 || j >= NX+2) expected = -1;
         if (schar_buf[i*array_of_sizes[1]+j] != expected) {
